unit MsCTF;

interface

uses
  Windows, SysUtils;

type
  PPWord = ^PWORD;

  TF_LANGUAGEPROFILE = record
    clsid: TGUID;
    langid: Word;
    catid: TGUID;
    fActive: Integer;
    guidProfile: TGUID;
  end;

  IEnumGUID = interface(IUnknown)
    ['{0002E000-0000-0000-C000-000000000046}']
    function Next(celt: UINT; out rgelt: TGUID; out pceltFetched: UINT): HResult; stdcall;
    function Skip(celt: UINT): HResult; stdcall;
    function Reset: HResult; stdcall;
    function Clone(out ppenum: IEnumGUID): HResult; stdcall;
  end;

  IEnumTfLanguageProfiles = interface(IUnknown)
    ['{3D61BF11-AC5F-42C8-A4CB-931BCC28C744}']
    function Clone(out ppenum: IEnumTfLanguageProfiles): HResult; stdcall;
    function Next(ulCount: LongWord; out pProfile: TF_LANGUAGEPROFILE; out pcFetch: LongWord): HResult; stdcall;
    function Reset: HResult; stdcall;
    function Skip(ulCount: LongWord): HResult; stdcall;
  end;

  PTfInputProcessorProfiles =  ^ITfInputProcessorProfiles;
  PPTfInputProcessorProfiles =  ^PTfInputProcessorProfiles;

  ITfInputProcessorProfiles = interface(IUnknown)
    ['{1F02B6C5-7842-4EE6-8A0B-9A24183A95CA}']
    function Register(var rclsid: TGUID): HResult; stdcall;
    function Unregister(var rclsid: TGUID): HResult; stdcall;
// !! "var word" -> PWideChar
    function AddLanguageProfile(var rclsid: TGUID; langid: Word; var guidProfile: TGUID;
                                pchDesc: PWideChar; cchDesc: LongWord; pchIconFile: PWideChar;
                                cchFile: LongWord; uIconIndex: LongWord): HResult; stdcall;
    function RemoveLanguageProfile(var rclsid: TGUID; langid: Word; var guidProfile: TGUID): HResult; stdcall;
    function EnumInputProcessorInfo(out ppenum: IEnumGUID): HResult; stdcall;
    function GetDefaultLanguageProfile(langid: Word; var catid: TGUID; out pclsid: TGUID;
                                       out pguidProfile: TGUID): HResult; stdcall;
    function SetDefaultLanguageProfile(langid: Word; var rclsid: TGUID; var guidProfiles: TGUID): HResult; stdcall;
    function ActivateLanguageProfile(var rclsid: TGUID; langid: Word; var guidProfiles: TGUID): HResult; stdcall;
    function GetActiveLanguageProfile(var rclsid: TGUID; out plangid: Word; out pguidProfile: TGUID): HResult; stdcall;
    function GetLanguageProfileDescription(var rclsid: TGUID; langid: Word; var guidProfile: TGUID;
                                           out pbstrProfile: WideString): HResult; stdcall;
    function GetCurrentLanguage(out plangid: Word): HResult; stdcall;
    function ChangeCurrentLanguage(langid: Word): HResult; stdcall;
    function GetLanguageList(ppLangId: PPWord; out pulCount: LongWord): HResult; stdcall;
    function EnumLanguageProfiles(langid: Word; out ppenum: IEnumTfLanguageProfiles): HResult; stdcall;
    function EnableLanguageProfile(var rclsid: TGUID; langid: Word; var guidProfile: TGUID;
                                   fEnable: Integer): HResult; stdcall;
    function IsEnabledLanguageProfile(var rclsid: TGUID; langid: Word; var guidProfile: TGUID;
                                      out pfEnable: Integer): HResult; stdcall;
    function EnableLanguageProfileByDefault(var rclsid: TGUID; langid: Word; var guidProfile: TGUID;
                                            fEnable: Integer): HResult; stdcall;
    function SubstituteKeyboardLayout(var rclsid: TGUID; langid: Word; var guidProfile: TGUID;
                                      HKL: HKL): HResult; stdcall;
  end;

  function TF_CreateInputProcessorProfiles(ppipr: PPTfInputProcessorProfiles): HRESULT; stdcall;
  function IsMSCTFAvailable: Boolean;

implementation


const
  MsCTFModName = 'Msctf.dll';
var
  MsCTFHandle: THandle = 0;


function TF_CreateInputProcessorProfiles; external MsCTFModName name 'TF_CreateInputProcessorProfiles';

function IsMSCTFAvailable: Boolean;
begin
  Result := MsCTFHandle <> 0;
end;

procedure InitMSCTF;
begin
  if MsCTFHandle = 0 then
    MsCTFHandle := SafeLoadLibrary(MsCTFModName);
end;

procedure DoneMSCTF;
begin
  if MsCTFHandle <> 0 then
    FreeLibrary(MsCTFHandle);
end;

initialization
  InitMSCTF;

finalization
  DoneMSCTF;

end.
